package com.scbfkj.uni.library;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.fasterxml.jackson.datatype.jdk8.Jdk8Module;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

public final class DataFormatUtil {
    private final static ObjectMapper objectMapper = new ObjectMapper();
    private final static String LOCAL_DATETIME_PATTERN = "yyyy-MM-dd HH:mm:ss";
    private final static DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(LOCAL_DATETIME_PATTERN);

    static {
        objectMapper.registerModule(new Jdk8Module());
        objectMapper.registerModule(new JavaTimeModule());
        objectMapper.setSerializationInclusion(JsonInclude.Include.ALWAYS);
    }

    public static JsonNode toJsonNode(Object source) throws JsonProcessingException {
        return objectMapper.readTree(toString(source));
    }

    public static <T> T stringToBean(String source, Class<T> clazz) throws JsonProcessingException {
        return objectMapper.readValue(source, clazz);
    }

    public static Map<?, ?> stringToMap(String source) throws JsonProcessingException {
        return stringToBean(source, Map.class);
    }

    public static ObjectNode stringToObjectNode(String source) throws JsonProcessingException {
        return objectMapper.readValue(source, ObjectNode.class);
    }

    public static ArrayNode stringToArrayNode(String source) throws JsonProcessingException {
        return objectMapper.readValue(source, ArrayNode.class);
    }

    public static String toString(Object source)  {
        if(Objects.isNull(source)) return null;
        if (source instanceof String || source instanceof Number)
            return source.toString();
        if (source instanceof LocalDate result) {
            return result.format(dateTimeFormatter);
        }
        if (source instanceof LocalDateTime result) {
            return result.format(dateTimeFormatter);
        }
        try {
            return objectMapper.writeValueAsString(source);
        } catch (JsonProcessingException e) {
            return source.toString();
        }
    }

    public static LocalDate toDate(Object dateObj) {
        if (dateObj instanceof String str) {
            return LocalDate.parse(str);
        } else if (dateObj instanceof LocalDate date) {
            return date;
        } else {
            return null;
        }
    }

    public static LocalDateTime toDateTime(Object dateTimeObj) {
        if (dateTimeObj instanceof String str) {
            return LocalDateTime.parse(str);
        } else if (dateTimeObj instanceof LocalDateTime dateTime) {
            return dateTime;
        } else {
            return null;
        }
    }

    public static LocalDate stringToDate(String dateStr) {
        return LocalDate.parse(dateStr);
    }

    public static LocalDateTime stringToDateTime(String dateTimeStr) {
        return LocalDateTime.parse(dateTimeStr);
    }

    public static List<?> toList(Object value) {
        if (Objects.isNull(value)) return null;
        ArrayList<Object> result = new ArrayList<>();
        if (value instanceof Iterable<?> iterable) {
            iterable.forEach(result::add);
            return result;
        }
        if (value.getClass().isArray()) {
            result.addAll(Arrays.asList(((Object[]) value)));
            return result;
        }
        return Collections.singletonList(value);
    }
    public static Object[] toArray(Object value) {
        if (Objects.isNull(value)) return null;

        return Objects.requireNonNull(toList(value)).toArray();
    }

    public static Map<?, ?> toMap(Object value) {
        if (Objects.isNull(value)) return null;
        if (value instanceof Map<?, ?> map) {
            return map;
        }
        if (value instanceof String str) {
            try {
                return DataFormatUtil.stringToMap(str);
            } catch (JsonProcessingException ignored) {

            }
        }
        return new HashMap<>() {{
            put("root", value);
        }};
    }
}
