package com.scbfkj.uni.library.script;

import com.fasterxml.jackson.databind.JsonNode;
import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.service.LoggerService;
import org.apache.commons.pool2.BaseKeyedPooledObjectFactory;
import org.apache.commons.pool2.KeyedObjectPool;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.apache.commons.pool2.impl.GenericKeyedObjectPool;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Objects;

public class JavaScriptEngineUtil {

    private static class JavaApply {
        private Object instance;
        private Method apply;
        private Method close;

        public JavaApply(Object instance, Method apply, Method close) {
            this.instance = instance;
            this.apply = apply;
            this.close = close;
        }

        public Object getInstance() {
            return instance;
        }

        public void setInstance(Object instance) {
            this.instance = instance;
        }

        public Method getApply() {
            return apply;
        }

        public void setApply(Method apply) {
            this.apply = apply;
        }

        public Method getClose() {
            return close;
        }

        public void setClose(Method close) {
            this.close = close;
        }

        public void invokeClose() throws InvocationTargetException, IllegalAccessException {
            if (Objects.nonNull(close)) {
                close.invoke(instance);
            }
        }

        public Object invokeApply(Object... args) throws InvocationTargetException, IllegalAccessException {
            return apply.invoke(instance, args);
        }
    }

    private final static KeyedObjectPool<String, JavaApply> javaRefPool = new GenericKeyedObjectPool<>(new BaseKeyedPooledObjectFactory<String, JavaApply>() {
        @Override
        public JavaApply create(String key) throws Exception {
            JsonNode jsonNode = DataFormatUtil.toJsonNode(key);
            JsonNode path = jsonNode.get("path");
            JsonNode className = jsonNode.get("className");
            JsonNode method = jsonNode.get("methodName");
            if (Objects.nonNull(path)) {
                File file = new File(System.getProperty("user.dir").concat(File.separator).concat("plugins").concat(File.separator).concat(path.asText()));
                if (!file.exists()) {
                    throw new RuntimeException("外部文件加载不存在：".concat(System.getProperty("user.dir")).concat(File.separator).concat("plugins").concat(File.separator).concat(path.asText()));
                }
                Method addURLMethod = null;

                boolean accessible = false;
                try {
                    addURLMethod = URLClassLoader.class.getDeclaredMethod("addURL", URL.class);
                    URLClassLoader classLoader = (URLClassLoader) ClassLoader.getSystemClassLoader();
                    accessible = addURLMethod.canAccess(null); //获取方法的访问权限以便恢复:判读是否可以访问: true  代表可以访问
                    if (!accessible) { //判读是否可以访问，如果是false，那就必须setAccessible(true)
                        addURLMethod.setAccessible(true);//实际上是为了提效
                    }
                    addURLMethod.invoke(classLoader, file.toURI().toURL());//加载到系统中
                } finally {
                    if (addURLMethod != null && !accessible) {
                        addURLMethod.setAccessible(false);//恢复原访问权限
                    }
                }
            }
            Class<?> classExample = Class.forName(className.asText()); //获取类实例
            Object classInstance = classExample.getConstructor().newInstance();//类实例接口 无参数构造
            Method javaMethod = null;
            Method closeMethod = null;
            for (Method currentMethod : classExample.getMethods()) {//循环所有方法
                String methodName = currentMethod.getName();
                if (methodName.equals(method.asText())) {
                    javaMethod = currentMethod;
                } else if (methodName.equals("close")) {
                    closeMethod = currentMethod;
                }
            }
            return new JavaApply(classInstance, javaMethod, closeMethod);
        }

        @Override
        public PooledObject<JavaApply> wrap(JavaApply value) {
            return new DefaultPooledObject<>(value);
        }

        @Override
        public void destroyObject(String key, PooledObject<JavaApply> p) throws Exception {
            JavaApply javaApply = p.getObject();
            javaApply.invokeClose();

        }
    });

    public static Map<String, Object> invoke(Map<String, Object> datasource, String expression, List<Object> args) {
        datasource.put("methodName", expression);
        String key = DataFormatUtil.toString(datasource);

        JavaApply javaApply = null;
        try {
            javaApply = javaRefPool.borrowObject(key);
            return UniReturnUtil.success(javaApply.invokeApply(args.toArray()));
        } catch (Exception e) {
            return UniReturnUtil.fail(e);
        } finally {
            if (Objects.nonNull(javaApply)) {
                try {
                    javaRefPool.returnObject(key, javaApply);
                } catch (Exception e) {
                    LoggerService.logSystemError(LocalDateTime.now(), null, UniReturnUtil.getMessage(e));
                }
            }
        }
    }
}
