package com.scbfkj.uni.process;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import org.apache.commons.pool2.BaseKeyedPooledObjectFactory;
import org.apache.commons.pool2.KeyedObjectPool;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.apache.commons.pool2.impl.GenericKeyedObjectPool;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.Producer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.springframework.util.StringUtils;

import java.time.Duration;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.ThreadPoolExecutor;

public class Kafka {
    private static final KeyedObjectPool<String, Producer<String, String>> producerPool = new GenericKeyedObjectPool<>(new BaseKeyedPooledObjectFactory<>() {
        @Override
        public Producer<String, String> create(String key) throws Exception {
            Map<String, Object> connectConfigMaps = (Map<String, Object>) DataFormatUtil.stringToMap(key);
            connectConfigMaps.put("request.timeout.ms", 60000);
            connectConfigMaps.put("max.request.size", 10240000);
            connectConfigMaps.put("compression.type", "gzip");
            connectConfigMaps.put("key.serializer", "org.apache.kafka.common.serialization.StringSerializer");
            connectConfigMaps.put("value.serializer", "org.apache.kafka.common.serialization.StringSerializer");

            return new KafkaProducer<>(connectConfigMaps);
        }

        @Override
        public PooledObject<Producer<String, String>> wrap(Producer<String, String> value) {
            return new DefaultPooledObject<>(value);
        }
    });
    private static final KeyedObjectPool<String, Consumer<String, String>> consumerPool = new GenericKeyedObjectPool<>(new BaseKeyedPooledObjectFactory<>() {
        @Override
        public Consumer<String, String> create(String key) throws Exception {
            Map<String, Object> connectConfigMaps = (Map<String, Object>) DataFormatUtil.stringToMap(key);
            if (!connectConfigMaps.containsKey("max.poll.records"))
                connectConfigMaps.put("max.poll.records", 1);
            connectConfigMaps.put("key.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
            connectConfigMaps.put("value.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
            return new KafkaConsumer<>(connectConfigMaps);
        }

        @Override
        public PooledObject<Consumer<String, String>> wrap(Consumer<String, String> value) {
            return new DefaultPooledObject<>(value);
        }
    });

    public static Map<String, Object> sendMessage(String connection, String topic, List<String> datas) throws Exception {

        if (Objects.isNull(datas) || datas.isEmpty()) {
            return UniReturnUtil.fail("数据为空");
        }

        List<Object> sendResult = new ArrayList<>();
        Producer<String, String> producer = null;
        try {
            producer = producerPool.borrowObject(connection);
//            多线程发送
            Producer<String, String> finalProducer = producer;
            datas.parallelStream().forEach(it -> {
                sendResult.add(finalProducer.send(new ProducerRecord<>(topic, DataFormatUtil.toString(it))));
            });
        } finally {
            if (Objects.nonNull(producer))
                producerPool.returnObject(connection, producer);
        }

        return UniReturnUtil.success(sendResult);
    }

    public static Map<String, Object> receptionMessage(String connectConfig, String sourceObjectName, String libraryId, String pollNumber, String fetch) throws Exception {
        Map<String, Object> connectConfigMaps = (Map<String, Object>) DataFormatUtil.stringToMap(connectConfig);
        if (!connectConfigMaps.containsKey("group.id")) {
            connectConfigMaps.put("group.id", "groupid" + (String.format("%s", libraryId)));
        }
        connectConfigMaps.put("max.poll.records", pollNumber == null ? 1 : Integer.parseInt(pollNumber));
        if (StringUtils.hasText(fetch)) {
            connectConfigMaps.put("max.partition.fetch.bytes", 11534336);
        }
        String key = DataFormatUtil.toString(connectConfig);
        Consumer<String, String> consumer = consumerPool.borrowObject(key);
        consumer.subscribe(Collections.singleton(sourceObjectName));
        ConsumerRecords<String, String> records = consumer.poll(Duration.ofSeconds(1));
        List<String> messageList = new ArrayList<>();
        if (records.count() > 0) {
            for (ConsumerRecord<String, String> record : records.records(sourceObjectName)) {
                String readValue = record.value().trim();
                if (!StringUtils.hasLength(readValue) || readValue.equals("{}")) {
                    continue;
                }
                messageList.add(readValue);
            }
        }
        consumerPool.returnObject(key, consumer);
        return UniReturnUtil.success(messageList);

    }
}
